@@ -9,6 +9,7 @@ check <- function(domain, condition, level=1, message="", call=sys.call(-1L)) {
     return(invisible(NULL))
 }
 
+
 ## SOCP - Example - 1
 ## min:  1 x1 + 1 x2 + 1 x3
 ## s.t.     x1 == sqrt(2)
@@ -21,14 +22,17 @@ test_cp_01 <- function(solver) {
     b <- c(sqrt(2))
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1), "soc"=list(2:4))
-    bound <- c(V_bound(li=1:3, lb=rep(-Inf, 3)), as.C_bound(cones))
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
+    bound <- V_bound(li = 1:3, lb = rep(-Inf, 3))
+
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(1), K_soc(3)), 
+                       rhs = c(b, h))
     x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
             bounds =  bound, maximum = FALSE)
 
-    opt <- ROI_solve(x, solver=solver)
+    opt <- ROI_solve(x, solver = solver)
+    
     check("CP-01@01", equal(sum(abs(opt$solution - c(sqrt(2), -1, -1))), 0))
     check("CP-01@02", equal(opt$objval, (sqrt(2) - 2)))
 }
@@ -48,12 +52,11 @@ test_cp_02 <- function(solver) {
     b <- c(sqrt(2), sqrt(2))
     G <- diag(x=-1, 6)
     h <- rep(0, 6)
-    cones <- list("free"=c(1, 2), "soc"=list(3:5, 6:8))
-    bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 6), bounds =  bound,
-            maximum = FALSE)
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_soc(c(3, 3))), 
+                       rhs = c(b, h))
+    x <- OP(objective = obj, constraints = lc)
 
     opt <- ROI_solve(x, solver=solver)
     check("CP-02@01", equal(sum(abs(opt$solution - c(sqrt(2), 1, 1, sqrt(2), 1, 1))), 0))
@@ -72,20 +75,19 @@ test_cp_03 <- function(solver) {
     A <- rbind(c(1, 0, 0),
                c(0, 1, 0))
     b <- c(1, 2)
-    G <- diag(x=-1, 3)
+    G <- -diag(3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "expp"=list(3:5))
-    bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
-            bounds =  bound, maximum = FALSE)
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_expp(1)), 
+                       rhs = c(b, h))
+    x <- OP(objective = obj, constraints = lc)
 
-    opt <- ROI_solve(x, solver=solver)
+    opt <- ROI_solve(x, solver = solver)
     check("CP-03@01", equal(opt$solution , c(1, 2, 2*exp(1/2))))
 }
 
-## EXPP - Example - 1
+## EXPP - Example - 2
 ## max:  x + y + z
 ## s.t.
 ## y e^(x/y) <= z
@@ -100,12 +102,11 @@ test_cp_04 <- function(solver) {
     b <- c(2, 2*exp(1/2))
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "expp"=list(3:5))
-    bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
-            bounds =  bound, maximum = TRUE)
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_expp(1)), 
+                       rhs = c(b, h))
+    x <- OP(objective = obj, constraints = lc, maximum = TRUE)
 
     opt <- ROI_solve(x, solver=solver)
     check("CP-04@01", equal(opt$solution , c(1, 2, 2*exp(1/2))))
@@ -126,14 +127,16 @@ test_cp_05 <- function(solver) {
     b <- c(1, exp(1))
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "expp"=list(3:5))
-    bound <- as.C_bound(cones)
+    ## cones <- list("free"=c(1, 2), "expp"=list(3:5))
+    ## bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
-            bounds =  bound, maximum = TRUE)
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_expp(1)), 
+                       rhs = c(b, h))
+    x <- OP(objective = obj, constraints = lc, 
+            types = rep("C", 3), maximum = TRUE)
 
-    opt <- ROI_solve(x, solver=solver)
+    opt <- ROI_solve(x, solver = solver)
     check("CP-05@01", equal(opt$solution , c(1, 1, exp(1))))
 }
 
@@ -146,20 +149,19 @@ test_cp_05 <- function(solver) {
 ##
 ##      c(-1, 1, exp(-2))
 test_cp_06 <- function(solver) {
-    obj <- c(1, 1, 1)
+    x <- OP(c(1, 1, 1))
     A <- rbind(c(1,  0, 0),
                c(0,  1, 0))
     b <- c(-1, 1)
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "expd"=list(3:5))
-    bound <- c(V_bound(li=1:3, lb=rep(-Inf, 3)), as.C_bound(cones))
-
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
-            bounds =  bound, maximum = FALSE)
+    constraints(x) <- C_constraint(L = rbind(A, G), 
+                                   cones = c(K_zero(2), K_expd(1)), 
+                                   rhs = c(b, h))
+    bounds(x) <- V_bound(li=1:3, lb=rep(-Inf, 3))
 
     opt <- ROI_solve(x, solver=solver)
+
     check("CP-06@01", equal(opt$solution , c(-1, 1, exp(-2))))
 }
 
@@ -179,12 +181,12 @@ test_cp_07 <- function(solver) {
     b <- c(4, 4)
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "powp"=list(list(i=3:5, a=0.5)))
-    bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), dir=rep("==", length(c(b, h))), rhs = c(b, h))
-    x <- OP(objective = obj, constraints = lc, types = rep("C", 3),
-            bounds =  bound, maximum = TRUE)
+    cc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_powp(0.5)), 
+                       rhs = c(b, h))
+    x <- OP(objective = obj, constraints = cc, 
+            types = rep("C", 3), maximum = TRUE)
 
     opt <- ROI_solve(x, solver=solver)
     check("CP-07@01", equal(opt$solution, c(4, 4, 4)))
@@ -207,15 +209,14 @@ test_cp_08 <- function(solver) {
     b <- c(2, 2)
     G <- diag(x=-1, 3)
     h <- rep(0, 3)
-    cones <- list("free"=c(1, 2), "powd"=list(list(i=3:5, a=0.5)))
-    bound <- as.C_bound(cones)
+    ## cones <- list("free"=c(1, 2), "powd"=list(list(i=3:5, a=0.5)))
+    ## bound <- as.C_bound(cones)
 
-    lc <- L_constraint(L = rbind(A, G), 
-                       dir=rep("==", length(c(b, h))), 
+    lc <- C_constraint(L = rbind(A, G), 
+                       cones = c(K_zero(2), K_powd(0.5)), 
                        rhs = c(b, h))
     x <- OP(objective = obj, constraints = lc, 
-            types = rep("C", 3), bounds =  bound,
-            maximum = TRUE)
+            types = rep("C", 3), maximum = TRUE)
 
     opt <- ROI_solve(x, solver=solver)
     check("CP-08@01", equal(opt$solution, c(2, 2, 4)))
@@ -249,11 +250,12 @@ test_cp_09 <- function(solver) {
     G2 <- vech(B1, B2, B3)
     h2 <- vech(b)
     h <- c(h1, h2)
-    bounds <- c(V_bound(li=1:3, lb=rep(-Inf, 3)), 
-                C_bound(1:3, type="psd"), C_bound(3+1:6, type="psd"))
+    bounds <- V_bound(li=1:3, lb=rep(-Inf, 3)) 
 
     x <- OP(objective = obj,
-            constraints = L_constraint(L = rbind(G1, G2), dir=rep("==", length(h)), rhs = h),
+            constraints = C_constraint(L = rbind(G1, G2), 
+                                       cones = K_psd(c(3, 6)), 
+                                       rhs = h),
             types = rep("C", length(obj)),
             bounds =  bounds,
             maximum = FALSE)
